/**
 * A class that implements the Fraction adt. Fractions are immutable.
 *  
 * @author Gloss
 * @version 1.1 - 12/Jul/2013 asantos
 **/
public class Fraction {

	/** 
	 * Fraction denominator 
	 **/
	private int num;

	/** 
	 * Fraction numerator 
	 **/
	private int den;

	/**
	 * Creates a new Fraction.
	 * 
	 * @param num - the fraction numerator.
	 * @param den - the fraction denominator.
	 * 
	 * @requires den != 0.
	 **/
	public Fraction(int num, int den) {
		this.num = num;
		this.den = den;
	}

	/**
	 * Returns the fraction numerator.
	 * 
	 * @return the fraction numerator.
	 **/
	public int num() {
		return num;
	}

	/**
	 * Returns the fraction denominator.
	 * 
	 * @return the fraction denominator.
	 **/
	public int den() {
		return den;
	}

	/**
	 * Add this fraction to another fraction and return the result.
	 * 
	 * @param other - the other fraction.
	 * @return the result of summing this fraction and other.
	 **/
	public Fraction sum(Fraction other) {
		return new Fraction(num * other.den + den * other.num, den * other.den);
	}

	/**
	 * Subtracts from this fraction another fraction and returns the result.
	 * 
	 * @param other - the other fraction.
	 * @return the result of subtracting other from fraction.
	 **/
	public Fraction sub(Fraction other) {
		return new Fraction(num * other.den - other.num * den, den * other.den);
	}

	/**
	 * Multiplies this fraction by another fraction and returns the result.
	 * 
	 * @param other - the other fraction.
	 * @return the result of multiplying this fraction and other.
	 **/
	public Fraction mult(Fraction other) {
		return new Fraction(num * other.num, den * other.den);
	}

	/**
	 * Divides this fraction by another fraction and returns the result.
	 * 
	 * @param other - the other fraction.
	 * @return the result of dividing this fraction and other fraction.
	 * 
	 * @requires !other.isZero()
	 **/
	public Fraction div(Fraction other) {
		return new Fraction(num * other.den, den * other.num);
	}

	/** 
	 * Invert current fraction.
	 * 
	 * @return the fraction inverted.
	 * @requires !isZero().
	 **/

	public Fraction inverse() {
		return new Fraction(den, num);
	}

	/**
	 * The fraction is Zero?
	 * 
	 * @return this.num == 0
	 **/
	public boolean isZero() {
		return num == 0;
	}

	/**
	 * Textual representation of the fraction.
	 * 
	 * @return the fraction String representation.
	 **/
	@Override
	public String toString() {
		return num + "/" + den;
	}

	/**
	 * Are the fractions equivalent?
	 * 
	 * @param f -the other fraction.
	 * 
	 * @return this.num * other.den == other.num * this.den;
	 **/
	public boolean areEquivalent(Fraction f) {
		return num * f.den == f.num * den;
	}

	/**
	 * Return the hash code for the fraction. 
	 * Equivalent fractions have the same hash code
	 * 
	 * @return the hash code for the fraction
	 */
	@Override
	public int hashCode() { 
		final int prime = 31;
		int result = 1;
		result = prime * result + (new Float(num/den)).hashCode();
		return result;
	}

	/**
	 * Compares this fraction to another object to test if they are equal.
	 * Considers equivalent fractions as equals.
	 * 
	 * @param other - the reference object with which to compare.
	 * @return true if this object is equal, otherwise return false.
	 **/
	@Override
	public boolean equals(Object other) {
		if (this == other)
			return true;
		if (other == null || !(other instanceof Fraction))
			return false;
		return areEquivalent((Fraction) other);
	}
}
