import java.util.Random;

/**
 * A class that implements the test for Fraction class. 
 * 
 * @author Gloss
 * @version 1.1 - 12/Jul/2013 asantos
 **/
public class FractionRandomTest {

	/**
	 * Our random object to generate random int.
	 */
	private Random rand;

	/**
	 * Fraction object.
	 */
	private Fraction fraction;

	/**
	 * How many tests do we want to perform.
	 */
	private int howMany;

	/**
	 * Max operations.
	 */
	private static int MAX_OPERATIONS = 8;
	
	/**
	 * Max value of integers.
	 */
	private static int MAX_INTEGER = 10;

	/**
	 * Constructor.
	 * 
	 * @param fraction - the fraction to be tested.
	 * @param howMany - How many tests we want to perform.
	 */
	public FractionRandomTest(int howMany) {
		this.rand = new Random();
		this.howMany = howMany;
		this.fraction = newFraction();
	}

	/**
	 * Runs the tests.
	 */
	public void run() {
		
		Fraction f = null;
		for (int i = 0; i < howMany; i++) {
		
			switch (rand.nextInt(MAX_OPERATIONS)) {
			// show the fraction numerator.
			case 0: 
				System.out.print("fraction num = " + fraction.num());
				break;

			// show the fraction denominator.
			case 1: 
				System.out.print("fraction den = " + fraction.den());
				break;
			
			// add the fraction with the other fraction.
			case 2: 
				f = newFraction();
				System.out.print(fraction + " sum " + f + " = " + fraction.sum(f));
				break;

			// multiplies the fraction with the other fraction.
			case 3: 
				f = newFraction();
				System.out.print(fraction + " mult " + f + " = " + fraction.mult(f));
				break;

			// are the fraction and other equivalent?
			case 4: 
				f = newFraction();
				System.out.print(fraction + " is" + (fraction.areEquivalent(f)? " ": " not ") +
						"equivalent to " + f );
				break;
			
			// divide fraction by other.
			case 5: 
				f = newFraction();
				if (f.num() != 0) {
					System.out.print(fraction + " div " + f + " = " + fraction.div(f));
				}
				break;

			// inverting fraction.
			case 6: 
				if (!fraction.isZero()) {
					System.out.print(fraction + " inverted = " + fraction.inverse());
				} else {
					System.out.print("Can't invert fraction with zero as numerator");
				}
				break;
			
			// The fractions are equal?
			case 7:
				f = newFraction();
				System.out.print(fraction + " is" + (fraction.equals(f)? " ": " not ") +
						"equal to " + f );
				break;
			}
			System.out.println();
		}
	}

	/**
	 * Creates a new fraction.
	 * 
	 * @return a new fraction.
	 **/
	private Fraction newFraction() {
		return new Fraction(newInt(), newNonZeroInt());
	}

	/**
	 * Creates a new integer.
	 * 
	 * @return the newly created integer.
	 **/
	private int newInt() {
		return rand.nextInt(MAX_INTEGER);
	}

	/**
	 * Creates a new non zero integer .
	 * 
	 * @return the newly created non zero integer.
	 */
	private int newNonZeroInt() {
		int result = newInt();
		return result == 0 ? 1 : result;
	}
}
